@@ -19,7 +19,7 @@ from octavia.amphorae.drivers import driver_base as driver_base
 LOG = logging.getLogger(__name__)
 
 
-class LoggingMixIn(driver_base.HealthMixin, driver_base.StatsMixin):
+class LoggingUpdate(object):
     def update_stats(self, stats):
         LOG.debug("Amphora %s no-op, update stats %s",
                   self.__class__.__name__, stats)
@@ -80,10 +80,11 @@ class NoopManager(object):
                   self.__class__.__name__, amphora.id)
         self.amphoraconfig[amphora.id] = (amphora.id, 'finalize amphora')
 
-    def post_network_plug(self, amphora):
-        LOG.debug("Amphora %s no-op, post network plug amphora %s",
-                  self.__class__.__name__, amphora.id)
-        self.amphoraconfig[amphora.id] = (amphora.id, 'post_network_plug')
+    def post_network_plug(self, amphora, port):
+        LOG.debug("Amphora %s no-op, post network plug amphora %s, port %s",
+                  self.__class__.__name__, amphora.id, port.id)
+        self.amphoraconfig[amphora.id, port.id] = (amphora.id, port.id,
+                                                   'post_network_plug')
 
     def post_vip_plug(self, load_balancer, amphorae_network_config):
         LOG.debug("Amphora %s no-op, post vip plug load balancer %s",
@@ -91,6 +92,12 @@ class NoopManager(object):
         self.amphoraconfig[(load_balancer.id, id(amphorae_network_config))] = (
             load_balancer.id, amphorae_network_config, 'post_vip_plug')
 
+    def upload_cert_amp(self, amphora, pem_file):
+        LOG.debug("Amphora %s no-op, upload cert amphora %s,with pem fle %s",
+                  self.__class__.__name__, amphora.id, pem_file)
+        self.amphoraconfig[amphora.id, pem_file] = (amphora.id, pem_file,
+                                                    'update_amp_cert_file')
+
 
 class NoopAmphoraLoadBalancerDriver(driver_base.AmphoraLoadBalancerDriver):
     def __init__(self):
@@ -125,10 +132,14 @@ class NoopAmphoraLoadBalancerDriver(driver_base.AmphoraLoadBalancerDriver):
 
         self.driver.finalize_amphora(amphora)
 
-    def post_network_plug(self, amphora):
+    def post_network_plug(self, amphora, port):
 
-        self.driver.post_network_plug(amphora)
+        self.driver.post_network_plug(amphora, port)
 
     def post_vip_plug(self, load_balancer, amphorae_network_config):
 
         self.driver.post_vip_plug(load_balancer, amphorae_network_config)
+
+    def upload_cert_amp(self, amphora, pem_file):
+
+        self.driver.upload_cert_amp(amphora, pem_file)
