@@ -16,20 +16,21 @@
 
 # make sure PYTHONPATH includes the home directory if you didn't install
 
-import logging
 import multiprocessing as multiproc
 import os
 import ssl
 import sys
 
 from oslo_config import cfg
+from oslo_reports import guru_meditation_report as gmr
 from werkzeug import serving
 
 from octavia.amphorae.backends.agent.api_server import server
 from octavia.amphorae.backends.health_daemon import health_daemon
 from octavia.common import service
+from octavia import version
+
 
-LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 CONF.import_group('amphora_agent', 'octavia.common.config')
 CONF.import_group('haproxy_amphora', 'octavia.common.config')
@@ -64,6 +65,8 @@ def main():
     # comment out to improve logging
     service.prepare_service(sys.argv)
 
+    gmr.TextGuruMeditation.setup_autorun(version)
+
     # Workaround for an issue with the auto-reload used below in werkzeug
     # Without it multiple health senders get started when werkzeug reloads
     if not os.environ.get('WERKZEUG_RUN_MAIN'):
