@@ -22,7 +22,7 @@ from oslo_db import options as db_options
 from oslo_log import log as logging
 import oslo_messaging as messaging
 
-
+from octavia.common import constants
 from octavia.common import utils
 from octavia.i18n import _LI
 from octavia import version
@@ -30,11 +30,11 @@ from octavia import version
 LOG = logging.getLogger(__name__)
 
 core_opts = [
-    cfg.StrOpt('bind_host', default='0.0.0.0',
-               help=_("The host IP to bind to")),
-    cfg.IntOpt('bind_port', default=9876,
-               help=_("The port to bind to")),
-    cfg.StrOpt('api_handler', default='simulated_handler',
+    cfg.IPOpt('bind_host', default='0.0.0.0',
+              help=_("The host IP to bind to")),
+    cfg.PortOpt('bind_port', default=9876,
+                help=_("The port to bind to")),
+    cfg.StrOpt('api_handler', default='queue_producer',
                help=_("The handler that the API communicates with")),
     cfg.StrOpt('api_paste_config', default="api-paste.ini",
                help=_("The API paste config file to use")),
@@ -54,27 +54,6 @@ core_opts = [
                       "means no limit")),
     cfg.StrOpt('host', default=utils.get_hostname(),
                help=_("The hostname Octavia is running on")),
-    cfg.StrOpt('nova_url',
-               default='http://127.0.0.1:8774/v2',
-               help=_('URL for connection to nova')),
-    cfg.StrOpt('nova_admin_username',
-               help=_('Username for connecting to nova in admin context')),
-    cfg.StrOpt('nova_admin_password',
-               help=_('Password for connection to nova in admin context'),
-               secret=True),
-    cfg.StrOpt('nova_admin_tenant_id',
-               help=_('The uuid of the admin nova tenant')),
-    cfg.StrOpt('nova_admin_auth_url',
-               default='http://localhost:5000/v2.0',
-               help=_('Authorization URL for connecting to nova in admin '
-                      'context')),
-    cfg.StrOpt('nova_ca_certificates_file',
-               help=_('CA file for novaclient to verify server certificates')),
-    cfg.BoolOpt('nova_api_insecure', default=False,
-                help=_("If True, ignore any SSL validation issues")),
-    cfg.StrOpt('os_region_name',
-               help=_('Region in Identity service catalog to use for '
-                      'communication with the OpenStack services.')),
     cfg.StrOpt('octavia_plugins',
                default='hot_plug_plugin',
                help=_('Name of the controller plugin to use'))
@@ -91,6 +70,10 @@ amphora_agent_opts = [
                default='/etc/network/interfaces.d/',
                help=_("The directory where new network interfaces "
                       "are located")),
+    cfg.StrOpt('agent_server_network_file',
+               help=_("The file where the network interfaces are located. "
+                      "Specifying this will override any value set for "
+                      "agent_server_network_dir.")),
     # Do not specify in octavia.conf, loaded at runtime
     cfg.StrOpt('amphora_id', help=_("The amphora ID.")),
 ]
@@ -106,12 +89,12 @@ networking_opts = [
 ]
 
 healthmanager_opts = [
-    cfg.StrOpt('bind_ip', default='0.0.0.0',
-               help=_('IP address the controller will listen on for '
-                      'heart beats')),
-    cfg.IntOpt('bind_port', default=5555,
-               help=_('Port number the controller will listen on'
-                      'for heart beats')),
+    cfg.IPOpt('bind_ip', default='0.0.0.0',
+              help=_('IP address the controller will listen on for '
+                     'heart beats')),
+    cfg.PortOpt('bind_port', default=5555,
+                help=_('Port number the controller will listen on'
+                       'for heart beats')),
     cfg.IntOpt('failover_threads',
                default=10,
                help=_('Number of threads performing amphora failovers.')),
@@ -134,16 +117,25 @@ healthmanager_opts = [
     # Used by the health manager on the amphora
     cfg.ListOpt('controller_ip_port_list',
                 help=_('List of controller ip and port pairs for the '
-                       'heartbeat receivers. Example [\'127.0.0.1:5555\', '
-                       '\'127.0.0.1:5555\']'),
+                       'heartbeat receivers. Example 127.0.0.1:5555, '
+                       '192.168.0.1:5555'),
                 default=[]),
     cfg.IntOpt('heartbeat_interval',
                default=10,
-               help=_('Sleep time between sending hearthbeats.'))
-]
+               help=_('Sleep time between sending hearthbeats.')),
+    cfg.StrOpt('event_streamer_driver',
+               help=_('Specifies which driver to use for the event_streamer '
+                      'for syncing the octavia and neutron_lbaas dbs. If you '
+                      'don\'t need to sync the database or are running '
+                      'octavia in stand alone mode use the '
+                      'noop_event_streamer'),
+               default='noop_event_streamer')]
 
 oslo_messaging_opts = [
     cfg.StrOpt('topic'),
+    cfg.StrOpt('event_stream_topic',
+               default='neutron_lbaas_event',
+               help=_('topic name for communicating events through a queue')),
 ]
 
 keystone_authtoken_v3_opts = [
@@ -154,13 +146,6 @@ keystone_authtoken_v3_opts = [
 ]
 
 haproxy_amphora_opts = [
-    cfg.StrOpt('username',
-               default='ubuntu',
-               help=_('Name of user for access to amphora.')),
-    cfg.StrOpt('key_path',
-               default='/opt/stack/.ssh/id_rsa',
-               help=_('Local absolute path to the private key '
-                      'loaded on amphora at boot.')),
     cfg.StrOpt('base_path',
                default='/var/lib/octavia',
                help=_('Base directory for amphora files.')),
@@ -173,26 +158,36 @@ haproxy_amphora_opts = [
                help=_('Retry threshold for connecting to amphorae.')),
     cfg.IntOpt('connection_retry_interval',
                default=5,
-               help=_('Retry timeout between attempts in seconds.')),
+               help=_('Retry timeout between connection attempts in '
+                      'seconds.')),
+    cfg.StrOpt('haproxy_stick_size', default='10k',
+               help=_('Size of the HAProxy stick table. Accepts k, m, g '
+                      'suffixes.  Example: 10k')),
 
     # REST server
-    cfg.StrOpt('bind_host', default='0.0.0.0',
-               help=_("The host IP to bind to")),
-    cfg.IntOpt('bind_port', default=9443,
-               help=_("The port to bind to")),
+    cfg.IPOpt('bind_host', default='0.0.0.0',
+              help=_("The host IP to bind to")),
+    cfg.PortOpt('bind_port', default=9443,
+                help=_("The port to bind to")),
     cfg.StrOpt('haproxy_cmd', default='/usr/sbin/haproxy',
                help=_("The full path to haproxy")),
     cfg.IntOpt('respawn_count', default=2,
                help=_("The respawn count for haproxy's upstart script")),
     cfg.IntOpt('respawn_interval', default=2,
                help=_("The respawn interval for haproxy's upstart script")),
-    cfg.StrOpt('haproxy_cert_dir', default='/tmp/',
-               help=_("The directory to store haproxy cert files in")),
+    cfg.FloatOpt('rest_request_conn_timeout', default=10,
+                 help=_("The time in seconds to wait for a REST API "
+                        "to connect.")),
+    cfg.FloatOpt('rest_request_read_timeout', default=60,
+                 help=_("The time in seconds to wait for a REST API "
+                        "response.")),
     # REST client
     cfg.StrOpt('client_cert', default='/etc/octavia/certs/client.pem',
                help=_("The client certificate to talk to the agent")),
     cfg.StrOpt('server_ca', default='/etc/octavia/certs/server_ca.pem',
                help=_("The ca which signed the server certificates")),
+    cfg.BoolOpt('use_upstart', default=True,
+                help=_("If False, use sysvinit.")),
 ]
 
 controller_worker_opts = [
@@ -201,16 +196,29 @@ controller_worker_opts = [
                help=_('Retry attempts to wait for Amphora to become active')),
     cfg.IntOpt('amp_active_wait_sec',
                default=10,
-               help=_('Seconds to wait for an Amphora to become active')),
+               help=_('Seconds to wait between checks on whether an Amphora '
+                      'has become active')),
     cfg.StrOpt('amp_flavor_id',
                default='',
                help=_('Nova instance flavor id for the Amphora')),
+    cfg.StrOpt('amp_image_tag',
+               default='',
+               help=_('Glance image tag for the Amphora image to boot. '
+                      'Use this option to be able to update the image '
+                      'without reconfiguring Octavia. '
+                      'Ignored if amp_image_id is defined.')),
     cfg.StrOpt('amp_image_id',
                default='',
+               deprecated_for_removal=True,
+               deprecated_reason='Superseded by amp_image_tag option.',
                help=_('Glance image id for the Amphora image to boot')),
     cfg.StrOpt('amp_ssh_key_name',
                default='',
                help=_('SSH key name used to boot the Amphora')),
+    cfg.BoolOpt('amp_ssh_access_allowed',
+                default=True,
+                help=_('Determines whether or not to allow access '
+                       'to the Amphorae')),
     cfg.StrOpt('amp_network',
                default='',
                help=_('Network to attach to the Amphora')),
@@ -231,7 +239,18 @@ controller_worker_opts = [
                help=_('Name of the network driver to use')),
     cfg.StrOpt('cert_generator',
                default='local_cert_generator',
-               help=_('Name of the cert generator to use'))
+               help=_('Name of the cert generator to use')),
+    cfg.StrOpt('loadbalancer_topology',
+               default=constants.TOPOLOGY_SINGLE,
+               choices=constants.SUPPORTED_LB_TOPOLOGIES,
+               help=_('Load balancer topology configuration. '
+                      'SINGLE - One amphora per load balancer. '
+                      'ACTIVE_STANDBY - Two amphora per load balancer.')),
+    cfg.BoolOpt('user_data_config_drive', default=False,
+                help=_('If True, build cloud-init user-data that is passed '
+                       'to the config drive on Amphora boot instead of '
+                       'personality files. If False, utilize personality '
+                       'files.'))
 ]
 
 task_flow_opts = [
@@ -247,11 +266,20 @@ core_cli_opts = []
 
 certificate_opts = [
     cfg.StrOpt('cert_manager',
-               default='local_cert_manager',
+               default='barbican_cert_manager',
                help='Name of the cert manager to use'),
     cfg.StrOpt('cert_generator',
                default='local_cert_generator',
                help='Name of the cert generator to use'),
+    cfg.StrOpt('barbican_auth',
+               default='barbican_acl_auth',
+               help='Name of the Barbican authentication method to use'),
+    cfg.StrOpt('region_name',
+               help='Region in Identity service catalog to use for '
+                    'communication with the barbican service.'),
+    cfg.StrOpt('endpoint_type',
+               default='publicURL',
+               help='The endpoint_type to be used for barbican service.')
 ]
 
 house_keeping_opts = [
@@ -266,10 +294,117 @@ house_keeping_opts = [
                help=_('DB cleanup interval in seconds')),
     cfg.IntOpt('amphora_expiry_age',
                default=604800,
-               help=_('Amphora expiry age in seconds'))
+               help=_('Amphora expiry age in seconds')),
+    cfg.IntOpt('cert_interval',
+               default=3600,
+               help=_('Certificate check interval in seconds')),
+    # 14 days for cert expiry buffer
+    cfg.IntOpt('cert_expiry_buffer',
+               default=1209600,
+               help=_('Seconds until certificate expiration')),
+    cfg.IntOpt('cert_rotate_threads',
+               default=10,
+               help=_('Number of threads performing amphora certificate'
+                      ' rotation'))
+]
+
+anchor_opts = [
+    cfg.StrOpt('url',
+               default='http://localhost:9999/v1/sign/default',
+               help=_('Anchor URL')),
+    cfg.StrOpt('username',
+               default='myusername',
+               help=_('Anchor username')),
+    cfg.StrOpt('password',
+               default='simplepassword',
+               help=_('Anchor password'),
+               secret=True)
+]
+
+keepalived_vrrp_opts = [
+    cfg.IntOpt('vrrp_advert_int',
+               default=1,
+               help=_('Amphora role and priority advertisement interval '
+                      'in seconds.')),
+    cfg.IntOpt('vrrp_check_interval',
+               default=5,
+               help=_('VRRP health check script run interval in seconds.')),
+    cfg.IntOpt('vrrp_fail_count',
+               default=2,
+               help=_('Number of successive failure before transition to a '
+                      'fail state.')),
+    cfg.IntOpt('vrrp_success_count',
+               default=2,
+               help=_('Number of successive failure before transition to a '
+                      'success state.')),
+    cfg.IntOpt('vrrp_garp_refresh_interval',
+               default=5,
+               help=_('Time in seconds between gratuitous ARP announcements '
+                      'from the MASTER.')),
+    cfg.IntOpt('vrrp_garp_refresh_count',
+               default=2,
+               help=_('Number of gratuitous ARP announcements to make on '
+                      'each refresh interval.'))
 
 ]
 
+nova_opts = [
+    cfg.StrOpt('service_name',
+               help=_('The name of the nova service in the keystone catalog')),
+    cfg.StrOpt('endpoint', help=_('A new endpoint to override the endpoint '
+                                  'in the keystone catalog.')),
+    cfg.StrOpt('region_name',
+               help=_('Region in Identity service catalog to use for '
+                      'communication with the OpenStack services.')),
+    cfg.StrOpt('endpoint_type', default='publicURL',
+               help=_('Endpoint interface in identity service to use')),
+    cfg.StrOpt('ca_certificates_file',
+               help=_('CA certificates file path')),
+    cfg.BoolOpt('insecure',
+                default=False,
+                help=_('Disable certificate validation on SSL connections ')),
+    cfg.BoolOpt('enable_anti_affinity', default=False,
+                help=_('Flag to indicate if nova anti-affinity feature is '
+                       'turned on.'))
+]
+
+neutron_opts = [
+    cfg.StrOpt('service_name',
+               help=_('The name of the neutron service in the '
+                      'keystone catalog')),
+    cfg.StrOpt('endpoint', help=_('A new endpoint to override the endpoint '
+                                  'in the keystone catalog.')),
+    cfg.StrOpt('region_name',
+               help=_('Region in Identity service catalog to use for '
+                      'communication with the OpenStack services.')),
+    cfg.StrOpt('endpoint_type', default='publicURL',
+               help=_('Endpoint interface in identity service to use')),
+    cfg.StrOpt('ca_certificates_file',
+               help=_('CA certificates file path')),
+    cfg.BoolOpt('insecure',
+                default=False,
+                help=_('Disable certificate validation on SSL connections ')),
+]
+
+glance_opts = [
+    cfg.StrOpt('service_name',
+               help=_('The name of the glance service in the '
+                      'keystone catalog')),
+    cfg.StrOpt('endpoint', help=_('A new endpoint to override the endpoint '
+                                  'in the keystone catalog.')),
+    cfg.StrOpt('region_name',
+               help=_('Region in Identity service catalog to use for '
+                      'communication with the OpenStack services.')),
+    cfg.StrOpt('endpoint_type', default='publicURL',
+               help=_('Endpoint interface in identity service to use')),
+    cfg.StrOpt('ca_certificates_file',
+               help=_('CA certificates file path')),
+    cfg.BoolOpt('insecure',
+                default=False,
+                help=_('Disable certificate validation on SSL connections ')),
+]
+
+
 # Register the configuration options
 cfg.CONF.register_opts(core_opts)
 cfg.CONF.register_opts(amphora_agent_opts, group='amphora_agent')
@@ -277,15 +412,21 @@ cfg.CONF.register_opts(networking_opts, group='networking')
 cfg.CONF.register_opts(oslo_messaging_opts, group='oslo_messaging')
 cfg.CONF.register_opts(haproxy_amphora_opts, group='haproxy_amphora')
 cfg.CONF.register_opts(controller_worker_opts, group='controller_worker')
+cfg.CONF.register_opts(keepalived_vrrp_opts, group='keepalived_vrrp')
 cfg.CONF.register_opts(task_flow_opts, group='task_flow')
 cfg.CONF.register_opts(oslo_messaging_opts, group='oslo_messaging')
 cfg.CONF.register_opts(house_keeping_opts, group='house_keeping')
+cfg.CONF.register_opts(anchor_opts, group='anchor')
 cfg.CONF.register_cli_opts(core_cli_opts)
 cfg.CONF.register_opts(certificate_opts, group='certificates')
 cfg.CONF.register_cli_opts(healthmanager_opts, group='health_manager')
 cfg.CONF.import_group('keystone_authtoken', 'keystonemiddleware.auth_token')
 cfg.CONF.register_opts(keystone_authtoken_v3_opts,
                        group='keystone_authtoken_v3')
+cfg.CONF.register_opts(nova_opts, group='nova')
+cfg.CONF.register_opts(glance_opts, group='glance')
+cfg.CONF.register_opts(neutron_opts, group='neutron')
+
 
 # Ensure that the control exchange is set correctly
 messaging.set_transport_defaults(control_exchange='octavia')
@@ -329,7 +470,7 @@ def setup_logging(conf):
 #         raise cfg.ConfigFilesNotFoundError(
 #             config_files=[cfg.CONF.api_paste_config])
 #     config_path = os.path.abspath(config_path)
-#     LOG.info(_("Config paste file: %s"), config_path)
+#     LOG.info(_LI("Config paste file: %s"), config_path)
 
 #     try:
 #         app = deploy.loadapp("config:%s" % config_path, name=app_name)
