@@ -14,14 +14,14 @@
 #    under the License.
 
 from cryptography.hazmat import backends
+from cryptography.hazmat.primitives import serialization
 from cryptography import x509
-from OpenSSL import crypto
-from OpenSSL import SSL
 from oslo_log import log as logging
 import six
 
 from octavia.common import data_models as data_models
 import octavia.common.exceptions as exceptions
+from octavia.i18n import _LE
 
 
 X509_BEG = "-----BEGIN CERTIFICATE-----"
@@ -43,31 +43,32 @@ def validate_cert(certificate, private_key=None,
     :param intermediates: PEM encoded intermediate certificates
     :returns: boolean
     """
-    x509 = _get_x509_from_pem_bytes(certificate)
+    cert = _get_x509_from_pem_bytes(certificate)
     if intermediates:
-        for x509pem in _split_x509s(intermediates):
-            _get_x509_from_pem_bytes(x509pem)
+        for x509Pem in _split_x509s(intermediates):
+            _get_x509_from_pem_bytes(x509Pem)
     if private_key:
-        pkey = _read_privatekey(
-            private_key, passphrase=private_key_passphrase)
-        ctx = SSL.Context(SSL.TLSv1_METHOD)
-        ctx.use_certificate(x509)
-        try:
-            ctx.use_privatekey(pkey)
-            ctx.check_privatekey()
-        except Exception:
+        pkey = _read_privatekey(private_key, passphrase=private_key_passphrase)
+        pknum = pkey.public_key().public_numbers()
+        certnum = cert.public_key().public_numbers()
+        if pknum != certnum:
             raise exceptions.MisMatchedKey
     return True
 
 
 def _read_privatekey(privatekey_pem, passphrase=None):
-    def _get_passphrase(*args):
-        if passphrase:
-            return six.b(passphrase)
-        else:
-            raise exceptions.NeedsPassphrase
-    return crypto.load_privatekey(crypto.FILETYPE_PEM, privatekey_pem,
-                                  _get_passphrase)
+    if passphrase:
+        if six.PY2:
+            passphrase = passphrase.encode("utf-8")
+        elif six.PY3:
+            passphrase = six.b(passphrase)
+
+    try:
+        pkey = privatekey_pem.encode('ascii')
+        return serialization.load_pem_private_key(pkey, passphrase,
+                                                  backends.default_backend())
+    except Exception:
+        raise exceptions.NeedsPassphrase
 
 
 def _split_x509s(xstr):
@@ -106,6 +107,7 @@ def get_host_names(certificate):
     """
     try:
         certificate = certificate.encode('ascii')
+
         cert = x509.load_pem_x509_certificate(certificate,
                                               backends.default_backend())
         cn = cert.subject.get_attributes_for_oid(x509.OID_COMMON_NAME)[0]
@@ -120,10 +122,27 @@ def get_host_names(certificate):
             host_names['dns_names'] = ext.value.get_values_for_type(
                 x509.DNSName)
         except x509.ExtensionNotFound:
-            LOG.debug("{0} extension not found".format(
-                x509.OID_SUBJECT_ALTERNATIVE_NAME))
+            LOG.debug("%s extension not found",
+                      x509.OID_SUBJECT_ALTERNATIVE_NAME)
 
         return host_names
+    except Exception:
+        LOG.exception(_LE("Unreadable certificate."))
+        raise exceptions.UnreadableCert
+
+
+def get_cert_expiration(certificate_pem):
+    """Extract the expiration date from the Pem encoded X509 certificate
+
+    :param certificate_pem: Certificate in PEM format
+    :returns: Expiration date of certificate_pem
+    """
+    try:
+        certificate = certificate_pem.encode('ascii')
+
+        cert = x509.load_pem_x509_certificate(certificate,
+                                              backends.default_backend())
+        return cert.not_valid_after
     except Exception as e:
         LOG.exception(e)
         raise exceptions.UnreadableCert
@@ -133,14 +152,16 @@ def _get_x509_from_pem_bytes(certificate_pem):
     """Parse X509 data from a PEM encoded certificate
 
     :param certificate_pem: Certificate in PEM format
-    :returns: pyOpenSSL high-level x509 data from the PEM string
+    :returns: crypto high-level x509 data from the PEM string
     """
     try:
-        x509 = crypto.load_certificate(crypto.FILETYPE_PEM,
-                                       certificate_pem)
+        certificate = certificate_pem.encode('ascii')
+
+        x509cert = x509.load_pem_x509_certificate(certificate,
+                                                  backends.default_backend())
     except Exception:
         raise exceptions.UnreadableCert
-    return x509
+    return x509cert
 
 
 def build_pem(tls_container):
@@ -155,7 +176,8 @@ def build_pem(tls_container):
         if tls_container.intermediates:
             pem = tls_container.intermediates[:]
         pem.extend([tls_container.certificate, tls_container.private_key])
-        return "\n".join(pem)
+
+        return '\n'.join(pem)
 
 
 def load_certificates_data(cert_mngr, listener):
@@ -168,12 +190,14 @@ def load_certificates_data(cert_mngr, listener):
 
         if listener.tls_certificate_id:
             tls_cert = _map_cert_tls_container(
-                cert_mngr.get_cert(listener.tls_certificate_id,
+                cert_mngr.get_cert(listener.project_id,
+                                   listener.tls_certificate_id,
                                    check_only=True))
         if listener.sni_containers:
             for sni_cont in listener.sni_containers:
                 cert_container = _map_cert_tls_container(
-                    cert_mngr.get_cert(sni_cont.tls_container.id,
+                    cert_mngr.get_cert(listener.project_id,
+                                       sni_cont.tls_container.id,
                                        check_only=True))
                 sni_certs.append(cert_container)
         return {'tls_cert': tls_cert, 'sni_certs': sni_certs}
