@@ -33,15 +33,17 @@ FAKE_SUBNET_INFO = {'subnet_cidr': FAKE_CIDR,
                     'gateway': FAKE_GATEWAY,
                     'mac_address': '123'}
 FAKE_UUID_1 = uuidutils.generate_uuid()
+FAKE_VRRP_IP = '10.1.0.1'
 
 
-class HaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
+class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
 
     def setUp(self):
-        super(HaproxyAmphoraLoadBalancerDriverTest, self).setUp()
+        super(TestHaproxyAmphoraLoadBalancerDriverTest, self).setUp()
         self.driver = driver.HaproxyAmphoraLoadBalancerDriver()
 
         self.driver.cert_manager = mock.MagicMock()
+        self.driver.cert_parser = mock.MagicMock()
         self.driver.client = mock.MagicMock()
         self.driver.jinja = mock.MagicMock()
 
@@ -52,9 +54,17 @@ class HaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.lb = self.sl.load_balancer
         self.port = network_models.Port(mac_address='123')
 
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
     @mock.patch('octavia.common.tls_utils.cert_parser.get_host_names')
-    def test_update(self, mock_cert):
-        mock_cert.return_value = {'cn': 'fakeCN'}
+    def test_update(self, mock_cert, mock_load_crt):
+        mock_cert.return_value = {'cn': 'aFakeCN'}
+        sconts = []
+        for sni_container in self.sl.sni_containers:
+            sconts.append(sni_container.tls_container)
+        mock_load_crt.return_value = {
+            'tls_cert': self.sl.default_tls_container,
+            'sni_certs': sconts
+        }
         self.driver.client.get_cert_md5sum.side_effect = [
             exc.NotFound, 'Fake_MD5', 'd41d8cd98f00b204e9800998ecf8427e']
         self.driver.jinja.build_config.side_effect = ['fake_config']
@@ -67,11 +77,22 @@ class HaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         # verify result
         # this is called 3 times
         self.driver.client.get_cert_md5sum.assert_called_with(
-            self.amp, self.sl.id, 'fakeCN.pem')
-        # this is called twice (last MD5 matches)
-        self.driver.client.upload_cert_pem.assert_called_with(
-            self.amp, self.sl.id, 'fakeCN.pem', six.b(''))
-        self.assertEqual(2, self.driver.client.upload_cert_pem.call_count)
+            self.amp, self.sl.id, 'aFakeCN.pem')
+        # this is called three times (last MD5 matches)
+        fp1 = ('--imainter1--\n\n--imainter1too--\n'
+               '\n--imapem1--\n\n--imakey1--\n')
+        fp2 = ('--imainter2--\n\n--imainter2too--\n'
+               '\n--imapem2--\n\n--imakey2--\n')
+        fp3 = ('--imainter3--\n\n--imainter3too--\n'
+               '\n--imapem3--\n\n--imakey3--\n')
+        ucp_calls = [
+            mock.call(self.amp, self.sl.id, 'aFakeCN.pem', fp1),
+            mock.call(self.amp, self.sl.id, 'aFakeCN.pem', fp2),
+            mock.call(self.amp, self.sl.id, 'aFakeCN.pem', fp3)
+        ]
+        self.driver.client.upload_cert_pem.assert_has_calls(ucp_calls,
+                                                            any_order=True)
+        self.assertEqual(3, self.driver.client.upload_cert_pem.call_count)
         # upload only one config file
         self.driver.client.upload_config.assert_called_once_with(
             self.amp, self.sl.id, 'fake_config')
@@ -91,6 +112,11 @@ class HaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.driver.client.start_listener.assert_called_once_with(
             self.amp, self.sl.id)
 
+    def test_upload_cert_amp(self):
+        self.driver.upload_cert_amp(self.amp, six.b('test'))
+        self.driver.client.update_cert_for_rotation.assert_called_once_with(
+            self.amp, six.b('test'))
+
     def test_stop(self):
         # Execute driver method
         self.driver.stop(self.sl, self.sv)
@@ -132,11 +158,16 @@ class HaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.driver.client.plug_network.assert_called_once_with(
             self.amp, dict(mac_address='123'))
 
+    def test_get_vrrp_interface(self):
+        self.driver.get_vrrp_interface(self.amp)
+        self.driver.client.get_interface.assert_called_once_with(
+            self.amp, self.amp.vrrp_ip)
 
-class AmphoraAPIClientTest(base.TestCase):
+
+class TestAmphoraAPIClientTest(base.TestCase):
 
     def setUp(self):
-        super(AmphoraAPIClientTest, self).setUp()
+        super(TestAmphoraAPIClientTest, self).setUp()
         self.driver = driver.AmphoraAPIClient()
         self.base_url = "https://127.0.0.1:9443/0.5"
         self.amp = models.Amphora(lb_network_ip='127.0.0.1', compute_id='123')
@@ -428,7 +459,7 @@ class AmphoraAPIClientTest(base.TestCase):
     def test_upload_invalid_cert_pem(self, m):
         m.put("{base}/listeners/{listener_id}/certificates/{filename}".format(
             base=self.base_url, listener_id=FAKE_UUID_1,
-            filename=FAKE_PEM_FILENAME), status_code=403)
+            filename=FAKE_PEM_FILENAME), status_code=400)
         self.assertRaises(exc.InvalidRequest, self.driver.upload_cert_pem,
                           self.amp, FAKE_UUID_1, FAKE_PEM_FILENAME,
                           "some_file")
@@ -460,6 +491,41 @@ class AmphoraAPIClientTest(base.TestCase):
                           self.amp, FAKE_UUID_1, FAKE_PEM_FILENAME,
                           "some_file")
 
+    @requests_mock.mock()
+    def test_update_cert_for_rotation(self, m):
+        m.put("{base}/certificate".format(base=self.base_url))
+        resp_body = self.driver.update_cert_for_rotation(self.amp,
+                                                         "some_file")
+        self.assertEqual(200, resp_body.status_code)
+
+    @requests_mock.mock()
+    def test_update_invalid_cert_for_rotation(self, m):
+        m.put("{base}/certificate".format(base=self.base_url), status_code=400)
+        self.assertRaises(exc.InvalidRequest,
+                          self.driver.update_cert_for_rotation, self.amp,
+                          "some_file")
+
+    @requests_mock.mock()
+    def test_update_cert_for_rotation_unauthorized(self, m):
+        m.put("{base}/certificate".format(base=self.base_url), status_code=401)
+        self.assertRaises(exc.Unauthorized,
+                          self.driver.update_cert_for_rotation, self.amp,
+                          "some_file")
+
+    @requests_mock.mock()
+    def test_update_cert_for_rotation_error(self, m):
+        m.put("{base}/certificate".format(base=self.base_url), status_code=500)
+        self.assertRaises(exc.InternalServerError,
+                          self.driver.update_cert_for_rotation, self.amp,
+                          "some_file")
+
+    @requests_mock.mock()
+    def test_update_cert_for_rotation_unavailable(self, m):
+        m.put("{base}/certificate".format(base=self.base_url), status_code=503)
+        self.assertRaises(exc.ServiceUnavailable,
+                          self.driver.update_cert_for_rotation, self.amp,
+                          "some_file")
+
     @requests_mock.mock()
     def test_get_cert_5sum(self, m):
         md5sum = {"md5sum": "some_real_sum"}
@@ -552,19 +618,24 @@ class AmphoraAPIClientTest(base.TestCase):
     def test_upload_config(self, m):
         config = {"name": "fake_config"}
         m.put(
-            "{base}/listeners/{listener_id}/haproxy".format(
-                base=self.base_url, listener_id=FAKE_UUID_1),
+            "{base}/listeners/{"
+            "amphora_id}/{listener_id}/haproxy".format(
+                amphora_id=self.amp.id, base=self.base_url,
+                listener_id=FAKE_UUID_1),
             json=config)
-        self.driver.upload_config(self.amp, FAKE_UUID_1, config)
+        self.driver.upload_config(self.amp, FAKE_UUID_1,
+                                  config)
         self.assertTrue(m.called)
 
     @requests_mock.mock()
     def test_upload_invalid_config(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/listeners/{listener_id}/haproxy".format(
-                base=self.base_url, listener_id=FAKE_UUID_1),
-            status_code=403)
+            "{base}/listeners/{"
+            "amphora_id}/{listener_id}/haproxy".format(
+                amphora_id=self.amp.id, base=self.base_url,
+                listener_id=FAKE_UUID_1),
+            status_code=400)
         self.assertRaises(exc.InvalidRequest, self.driver.upload_config,
                           self.amp, FAKE_UUID_1, config)
 
@@ -572,8 +643,10 @@ class AmphoraAPIClientTest(base.TestCase):
     def test_upload_config_unauthorized(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/listeners/{listener_id}/haproxy".format(
-                base=self.base_url, listener_id=FAKE_UUID_1),
+            "{base}/listeners/{"
+            "amphora_id}/{listener_id}/haproxy".format(
+                amphora_id=self.amp.id, base=self.base_url,
+                listener_id=FAKE_UUID_1),
             status_code=401)
         self.assertRaises(exc.Unauthorized, self.driver.upload_config,
                           self.amp, FAKE_UUID_1, config)
@@ -582,8 +655,10 @@ class AmphoraAPIClientTest(base.TestCase):
     def test_upload_config_server_error(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/listeners/{listener_id}/haproxy".format(
-                base=self.base_url, listener_id=FAKE_UUID_1),
+            "{base}/listeners/{"
+            "amphora_id}/{listener_id}/haproxy".format(
+                amphora_id=self.amp.id, base=self.base_url,
+                listener_id=FAKE_UUID_1),
             status_code=500)
         self.assertRaises(exc.InternalServerError, self.driver.upload_config,
                           self.amp, FAKE_UUID_1, config)
@@ -592,8 +667,10 @@ class AmphoraAPIClientTest(base.TestCase):
     def test_upload_config_service_unavailable(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/listeners/{listener_id}/haproxy".format(
-                base=self.base_url, listener_id=FAKE_UUID_1),
+            "{base}/listeners/{"
+            "amphora_id}/{listener_id}/haproxy".format(
+                amphora_id=self.amp.id, base=self.base_url,
+                listener_id=FAKE_UUID_1),
             status_code=503)
         self.assertRaises(exc.ServiceUnavailable, self.driver.upload_config,
                           self.amp, FAKE_UUID_1, config)
@@ -613,3 +690,36 @@ class AmphoraAPIClientTest(base.TestCase):
         )
         self.driver.plug_network(self.amp, self.port_info)
         self.assertTrue(m.called)
+
+    @requests_mock.mock()
+    def test_upload_vrrp_config(self, m):
+        config = '{"name": "bad_config"}'
+        m.put("{base}/vrrp/upload".format(
+            base=self.base_url)
+        )
+        self.driver.upload_vrrp_config(self.amp, config)
+        self.assertTrue(m.called)
+
+    @requests_mock.mock()
+    def test_vrrp_action(self, m):
+        action = 'start'
+        m.put("{base}/vrrp/{action}".format(base=self.base_url, action=action))
+        self.driver._vrrp_action(action, self.amp)
+        self.assertTrue(m.called)
+
+    @requests_mock.mock()
+    def test_get_interface(self, m):
+        interface = [{"interface": "eth1"}]
+        ip_addr = '10.0.0.1'
+        m.get("{base}/interface/{ip_addr}".format(base=self.base_url,
+                                                  ip_addr=ip_addr),
+              json=interface)
+        self.driver.get_interface(self.amp, ip_addr)
+        self.assertTrue(m.called)
+
+        m.register_uri('GET',
+                       self.base_url + '/interface/' + ip_addr,
+                       status_code=500, reason='FAIL', json='FAIL')
+        self.assertRaises(exc.InternalServerError,
+                          self.driver.get_interface,
+                          self.amp, ip_addr)
